/*****************************************************************************

	Fox Counter for METAL GEAR SOLID PEACE WALKER

	Copyright (C) 2012 Yumimint

*****************************************************************************/
#include <windows.h>
#include <commctrl.h>
#ifdef _MSC_VER
#	pragma comment (lib, "comctl32.lib")
#endif

#include <tchar.h>
#include <cstdio>
#include <cassert>
#include "resource.h"

enum {
#	define X(a, b) IDX_##a,
#	include "table.h"
#	undef X
};

static const int CATEGORY_NUM(11);
static const int COUNTER_NUM(11+2);

static HWND   s_edit;
static HBRUSH s_hbrEdit1;
static HBRUSH s_hbrEdit2;
static int   s_addvalue = 1;
static int   s_maxnum;
static int   s_counter      [COUNTER_NUM] = {0};
static int   s_counter_fixed[COUNTER_NUM] = {0};
static float s_rate[CATEGORY_NUM];

static const char DATANAME[] = "FoxCounter.txt";

static const char *const catname[] = {
#	define X(a, b) b,
#	include "table.h"
#	undef X
};

static LRESULT CALLBACK dialog_proc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static LRESULT command(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void update(HWND hwnd, bool button_enable);
static void load();
static void save();
static void log_clear();
static void log_vprintf(const char *fmt, va_list arglist);
static void log_printf(const char *fmt, ...);

/*****************************************************************************
 *	main
 */
int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	InitCommonControls();
	DialogBox(hInstance, MAKEINTRESOURCE(IDD_DIALOG1), NULL, (DLGPROC) dialog_proc);
	return 0;
}

/**
 */
static LRESULT CALLBACK dialog_proc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
	case WM_INITDIALOG:
		s_edit = GetDlgItem(hwnd, IDC_EDIT_LOG);
		s_hbrEdit1 = CreateSolidBrush(RGB(128, 255, 128));
		s_hbrEdit2 = CreateSolidBrush(RGB(255, 128, 128));
		SendMessage(GetDlgItem(hwnd, IDC_RADIO1), BM_CLICK, 0, 0);
		for (int i(0); i < CATEGORY_NUM; ++i) {
			SendDlgItemMessageA(hwnd, IDC_CAT1+i, WM_SETTEXT, 0, (LPARAM) catname[i]);
		}
		load();
		update(hwnd, false);
		return 1;

	case WM_DESTROY:
		DeleteObject(s_hbrEdit1);
		DeleteObject(s_hbrEdit2);
		return 0;

	case WM_CTLCOLORSTATIC:
		for (int i(0); i < CATEGORY_NUM; ++i) {
			if ((HWND)lParam == GetDlgItem(hwnd, IDC_EDIT_R1+i)) {
				if (s_rate[i] <= 1.1f && s_rate[i] >= 0.9f) {
					SetBkMode((HDC) wParam, TRANSPARENT);
					SetTextColor((HDC) wParam, RGB(0, 0, 0));
					return (LRESULT) s_hbrEdit1;
				}
			}
			if ((HWND)lParam == GetDlgItem(hwnd, IDC_EDIT_C1+i)) {
				if (s_counter[i] == s_maxnum) {
					SetBkMode((HDC) wParam, TRANSPARENT);
					SetTextColor((HDC) wParam, RGB(255, 255, 0));
					return (LRESULT) s_hbrEdit2;
				}
			}
		}
		break;

	case WM_COMMAND:
		return command(hwnd, msg, wParam, lParam);
	}
	return 0;
}

/**
 */
static LRESULT command(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam)) {
	case IDCANCEL:
	  EndDialog(hwnd, 0);
	  break;

	case IDC_BUTTON_FIX:
		for (int i(0); i < COUNTER_NUM; ++i) {
			s_counter_fixed[i] = s_counter[i];
		}
		update(hwnd, false);
		save();
		log_printf("---- m ----\n");
		break;

	case IDC_BUTTON_DISCARD:
		for (int i(0); i < COUNTER_NUM; ++i) {
			s_counter[i] = s_counter_fixed[i];
		}
		update(hwnd, false);
		log_printf("---- LZ ----\n");
		break;
	
	case IDC_BUTTON_CLEAR:
		log_clear();
		break;

	case IDC_RADIO1:
		s_addvalue = 1;
		EnableWindow(GetDlgItem(hwnd, IDC_EDIT_VALUE), false);
		break;

	case IDC_RADIO2:
		EnableWindow(GetDlgItem(hwnd, IDC_EDIT_VALUE), true);
		{
			char buf[32];
			GetWindowTextA(GetDlgItem(hwnd, IDC_EDIT_VALUE), buf, sizeof(buf));
			s_addvalue = atoi(buf);
		}
		break;

	case IDC_EDIT_VALUE:
		if (HIWORD(wParam) == EN_UPDATE) {
			char buf[32];
			GetWindowTextA(GetDlgItem(hwnd, IDC_EDIT_VALUE), buf, sizeof(buf));
			s_addvalue = atoi(buf);
		}
		break;

	default:
		if ((LOWORD(wParam) >= IDC_BUTTON_A1 && LOWORD(wParam) <= IDC_BUTTON_AB) ||
			(LOWORD(wParam) >= IDC_BUTTON_B1 && LOWORD(wParam) <= IDC_BUTTON_BB))
		{
			int i;
			if ((LOWORD(wParam) >= IDC_BUTTON_A1 && LOWORD(wParam) <= IDC_BUTTON_AB)) {
				i = LOWORD(wParam) - IDC_BUTTON_A1;
				s_counter[IDX_NOKILL] += s_addvalue;
				log_printf("%s ͉ +%d\n", catname[i], s_addvalue);
			} else {
				i = LOWORD(wParam) - IDC_BUTTON_B1;
				s_counter[IDX_KILL  ] += s_addvalue;
				log_printf("%s EQ +%d\n", catname[i], s_addvalue);
			}
			s_counter[i] += s_addvalue;
			update(hwnd, true);
		}
	  break;
	}
	return 1;
}


/**
 *		\XV
 */
static void update(HWND hwnd, bool button_enable)
{
	double sum(0);

	s_maxnum = 1;
	for (int i(0); i < CATEGORY_NUM; ++i) {
		if (s_maxnum < s_counter[i])
			s_maxnum = s_counter[i];
		sum += s_counter[i];
	}
	double avg(sum / CATEGORY_NUM);
	if (avg == 0.0) avg = 1;

	for (int i(0); i < CATEGORY_NUM; ++i) {
		char buf[32];
		sprintf(buf, "%d", s_counter[i]);
		SendDlgItemMessageA(hwnd, IDC_EDIT_C1 + i, WM_SETTEXT, 0, (LPARAM) buf);

		s_rate[i] = s_counter[i] / avg;

		sprintf(buf, "%.4f", s_counter[i] / avg);
		SendDlgItemMessageA(hwnd, IDC_EDIT_R1 + i, WM_SETTEXT, 0, (LPARAM) buf);
	}

	char buf[32];
	sprintf(buf, "%d", s_counter[IDX_NOKILL]);
	SendDlgItemMessageA(hwnd, IDC_EDIT_NOKILL, WM_SETTEXT, 0, (LPARAM) buf);

	sprintf(buf, "%d", s_counter[IDX_KILL]*2);
	SendDlgItemMessageA(hwnd, IDC_EDIT_KILL, WM_SETTEXT, 0, (LPARAM) buf);

	sprintf(buf, "%d", s_counter[IDX_NOKILL] - s_counter[IDX_KILL]*2);
	SendDlgItemMessageA(hwnd, IDC_EDIT_DIFF, WM_SETTEXT, 0, (LPARAM) buf);

	EnableWindow(GetDlgItem(hwnd, IDC_BUTTON_DISCARD), button_enable);
	EnableWindow(GetDlgItem(hwnd, IDC_BUTTON_FIX    ), button_enable);
}

/**
 */
static void load()
{
	FILE *fp = fopen(DATANAME, "rt");
	if (!fp) return;
		char buf[1024];
	int i(0);
	while (fgets(buf, sizeof(buf), fp)) {
		int v(0);
		sscanf(buf, "%d", &v);
		s_counter[i] = s_counter_fixed[i] = v;
		++i;
	}
	fclose(fp);
}

/**
 */
static void save()
{
	FILE *fp = fopen(DATANAME, "wt");
	if (!fp) return;
	for (int i(0); i < COUNTER_NUM; ++i) {
		fprintf(fp, "%d %s\n", s_counter[i], catname[i]);
	}
	fclose(fp);
}

/**
 */
static void log_clear()
{
	if (!s_edit) return;
	SendMessage(s_edit, EM_SETSEL, 0, -1);
	SendMessage(s_edit, EM_REPLACESEL, 0, 0);
	SendMessage(s_edit, WM_PAINT, 0, 0);
}

/**
 */
static void log_vprintf(const char *fmt, va_list arglist)
{
	if (!s_edit) return;
	char buf[1024];
	vsnprintf(buf, sizeof(buf), fmt, arglist);
	WPARAM lSize = SendMessage(s_edit, WM_GETTEXTLENGTH, 0, 0);
	SendMessage(s_edit, EM_SETSEL, lSize, lSize);
	SendMessageA(s_edit, EM_REPLACESEL, 0, (LPARAM) buf);
}

/**
 */
static void log_printf(const char *fmt, ...)
{
	va_list arglist; 
	va_start(arglist, fmt); 
	log_vprintf(fmt, arglist);
	va_end(arglist); 
}

